<?php
/**
 * Plugin Name: Naver Blog Reviews
 * Plugin URI: https://url.kr/p/wp-plugin/naver-blog-reviews/
 * Description: 네이버 블로그 리뷰 플러그인은 각 포스트에 대한 관련 네이버 블로그 리뷰를 자동으로 가져와 표시합니다. 포스트 하단에 최대 5개의 관련 블로그 리뷰가 표시됩니다.
 * Version: 2.6
 * Author: 호텔천사
 * Author URI: https://url.kr
 */

// ===================================
// 초기 설정
// ===================================

/* 직접 접근 방지 */
if (!defined('ABSPATH')) {
    exit;
}

// ===================================
// 메인 클래스
// ===================================

class NaverBlogReviews {
    private $options;
    private $transient_prefix = 'nbr_cache_';
    private $cache_time = 3600; // 1 hour
    private $meta_key = 'naver_search_keyword'; // Custom field for Naver search keyword

    /**
     * 클래스 생성자 - 필요한 액션과 필터 등록
     */
    public function __construct() {
        // 메인 관리자 메뉴 및 서브메뉴 추가
        add_action('admin_menu', array($this, 'add_admin_menu'));
        
        add_action('admin_init', array($this, 'page_init'));
        add_filter('the_content', array($this, 'append_naver_reviews'));
        add_action('wp_enqueue_scripts', array($this, 'enqueue_styles'));
        add_action('add_meta_boxes', array($this, 'add_meta_box'));
        add_action('save_post', array($this, 'save_meta_box_data'));
        
        // 네이버 검색 키워드 메타박스가 화면 옵션에서 숨겨지지 않도록 설정
        add_filter('default_hidden_meta_boxes', array($this, 'ensure_meta_box_is_visible'), 10, 2);
        // 관리자 페이지에서 필요한 스타일과 스크립트 로드
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));
        
        $this->options = get_option('naver_blog_reviews_options');
    }

    // ===================================
    // 관리자 메뉴 설정
    // ===================================
    
    /**
     * 관리자 메뉴에 플러그인 메뉴 및 서브메뉴 추가
     */
    public function add_admin_menu() {
        // 메인 메뉴 추가
        add_menu_page(
            '네이버 블로그 리뷰',         // 페이지 제목
            '네이버 블로그 리뷰',         // 메뉴 제목
            'manage_options',            // 필요 권한
            'naver-blog-reviews',        // 메뉴 슬러그
            array($this, 'create_admin_page'), // 콜백 함수
            'dashicons-search',          // 아이콘
            30                           // 메뉴 위치
        );
        
        // 설정 서브메뉴 추가
        add_submenu_page(
            'naver-blog-reviews',        // 부모 메뉴 슬러그
            'API 설정',                  // 페이지 제목
            'API 설정',                  // 메뉴 제목
            'manage_options',            // 필요 권한
            'naver-blog-reviews',        // 메뉴 슬러그 (부모와 같게 설정하면 기본 서브메뉴가 됨)
            array($this, 'create_admin_page') // 콜백 함수
        );
        
        // 사용설명서 서브메뉴 추가
        add_submenu_page(
            'naver-blog-reviews',        // 부모 메뉴 슬러그
            '사용설명서',                // 페이지 제목
            '사용설명서',                // 메뉴 제목
            'manage_options',            // 필요 권한
            'naver-blog-reviews-manual', // 메뉴 슬러그
            array($this, 'display_manual') // 콜백 함수
        );
    }

    /**
     * 사용설명서 페이지 표시
     */
    public function display_manual() {
        // readme.txt 파일 경로
        $readme_file = plugin_dir_path(__FILE__) . 'readme.txt';
        $manual_content = '';
        
        // readme.txt 파일 존재 여부 확인
        if (file_exists($readme_file)) {
            // 파일 내용 읽기
            $manual_content = file_get_contents($readme_file);
            
            // 기본적인 마크다운 형식 HTML로 변환
            $manual_content = $this->format_manual_content($manual_content);
        } else {
            $manual_content = '<p>사용설명서 파일(readme.txt)을 찾을 수 없습니다.</p>';
        }
        
        // 사용설명서 페이지 출력
        ?>
        <div class="wrap">
            <h1>네이버 블로그 리뷰 - 사용설명서</h1>
            <div class="card" style="max-width: 800px; padding: 20px; margin-top: 20px;">
                <?php echo $manual_content; ?>
            </div>
        </div>
        <?php
    }
    
    /**
     * readme.txt 내용을 HTML로 포맷팅
     * 
     * @param string $content readme.txt 파일 내용
     * @return string 포맷팅된 HTML
     */
    private function format_manual_content($content) {
        // 줄바꿈을 <br> 태그로 변환
        $content = nl2br($content);
        
        // 제목 포맷팅
        $content = preg_replace('/===\s*(.*?)\s*===/s', '<h2>$1</h2>', $content);
        $content = preg_replace('/==\s*(.*?)\s*==/s', '<h3>$1</h3>', $content);
        $content = preg_replace('/=\s*(.*?)\s*=/s', '<h4>$1</h4>', $content);
        
        // 리스트 아이템 포맷팅
        $content = preg_replace('/^\s*-\s*(.*?)$/m', '<li>$1</li>', $content);
        $content = preg_replace('/(<li>.*?<\/li>\s*)+/s', '<ul>$0</ul>', $content);
        
        // 버전 정보 블록 스타일링
        $content = preg_replace('/<h4>([\d\.]+)<\/h4>/s', '<h4 style="background-color: #f8f9fa; padding: 10px; border-left: 4px solid #007bff;">버전 $1</h4>', $content);
        
        return $content;
    }

    /**
     * 플러그인 관리자 페이지 생성
     */
    public function create_admin_page() {
        ?>
        <div class="wrap">
            <h1>네이버 블로그 리뷰 설정</h1>
            <p>네이버 API를 사용하여 블로그 리뷰를 가져오기 위한 설정을 입력해주세요.</p>
            <form method="post" action="options.php">
            <?php
                settings_fields('naver_blog_reviews_options_group');
                do_settings_sections('naver-blog-reviews-admin');
                submit_button('설정 저장');
            ?>
            </form>
        </div>
        <?php
    }

    /**
     * 관리자 페이지 초기화 - 설정 필드 등록
     */
    public function page_init() {
        register_setting(
            'naver_blog_reviews_options_group',
            'naver_blog_reviews_options',
            array($this, 'sanitize')
        );

        add_settings_section(
            'naver_blog_reviews_setting_section',
            '네이버 API 설정',
            array($this, 'section_info'),
            'naver-blog-reviews-admin'
        );

        add_settings_field(
            'client_id',
            '클라이언트 ID',
            array($this, 'client_id_callback'),
            'naver-blog-reviews-admin',
            'naver_blog_reviews_setting_section'
        );

        add_settings_field(
            'client_secret',
            '클라이언트 시크릿',
            array($this, 'client_secret_callback'),
            'naver-blog-reviews-admin',
            'naver_blog_reviews_setting_section'
        );
    }

    /**
     * 입력값 sanitize 처리
     * 
     * @param array $input 사용자 입력값
     * @return array 정제된 입력값
     */
    public function sanitize($input) {
        $sanitized_input = array();
        if (isset($input['client_id']))
            $sanitized_input['client_id'] = sanitize_text_field($input['client_id']);
        if (isset($input['client_secret']))
            $sanitized_input['client_secret'] = sanitize_text_field($input['client_secret']);
        return $sanitized_input;
    }

    /**
     * 섹션 설명 정보 표시
     */
    public function section_info() {
        echo '네이버 개발자 센터(https://developers.naver.com)에서 발급받은 API 인증 정보를 입력하세요:';
    }

    /**
     * Client ID 필드 렌더링
     */
    public function client_id_callback() {
        ?>
        <div class="input-group" style="display: flex; max-width: 400px;">
            <span class="input-group-text" style="display: flex; align-items: center; padding: 6px 12px; border: 1px solid #ccc; border-radius: 4px 0 0 4px; border-right: none; background-color: #fff;">
                <i class="dashicons dashicons-admin-network"></i>
            </span>
            <input type="text" id="client_id" name="naver_blog_reviews_options[client_id]" 
                   value="<?php echo isset($this->options['client_id']) ? esc_attr($this->options['client_id']) : ''; ?>" 
                   class="regular-text" style="flex: 1; padding: 6px 12px; border: 1px solid #ccc; border-radius: 0 4px 4px 0; border-left: none;" 
                   placeholder="네이버 API 클라이언트 ID 입력">
        </div>
        <?php
    }

    /**
     * Client Secret 필드 렌더링
     */
    public function client_secret_callback() {
        ?>
        <div class="input-group" style="display: flex; max-width: 400px;">
            <span class="input-group-text" style="display: flex; align-items: center; padding: 6px 12px; border: 1px solid #ccc; border-radius: 4px 0 0 4px; border-right: none; background-color: #fff;">
                <i class="dashicons dashicons-lock"></i>
            </span>
            <input type="text" id="client_secret" name="naver_blog_reviews_options[client_secret]" 
                   value="<?php echo isset($this->options['client_secret']) ? esc_attr($this->options['client_secret']) : ''; ?>" 
                   class="regular-text" style="flex: 1; padding: 6px 12px; border: 1px solid #ccc; border-radius: 0 4px 4px 0; border-left: none;" 
                   placeholder="네이버 API 클라이언트 시크릿 입력">
        </div>
        <?php
    }

    // ===================================
    // 스타일 및 스크립트 로드
    // ===================================

    /**
     * 프론트엔드에 필요한 스타일 로드
     */
    public function enqueue_styles() {
        wp_enqueue_style('bootstrap-icons', 'https://cdn.jsdelivr.net/npm/bootstrap-icons@1.11.0/font/bootstrap-icons.css');
        wp_enqueue_style('naver-blog-reviews', plugins_url('naver-blog-reviews.css', __FILE__));
    }

    /**
     * 관리자 페이지에 필요한 스크립트와 스타일을 로드
     * 
     * @param string $hook 현재 관리자 페이지 hook
     */
    public function enqueue_admin_scripts($hook) {
        // 포스트 편집 화면일 때만 로드
        if ($hook == 'post.php' || $hook == 'post-new.php') {
            // Bootstrap Icons 로드
            wp_enqueue_style('bootstrap-icons', 'https://cdn.jsdelivr.net/npm/bootstrap-icons@1.11.0/font/bootstrap-icons.css');
            
            // 메타박스 스타일을 위한 인라인 CSS 추가
            wp_add_inline_style('wp-admin', '
                /* Input group styles for meta box */
                .input-group {
                    display: flex;
                    margin-bottom: 10px;
                }
                .input-group-text {
                    border-right: none;
                    background-color: #fff;
                    display: flex;
                    align-items: center;
                    padding: 6px 12px;
                    border: 1px solid #ccc;
                    border-radius: 4px 0 0 4px;
                }
                .form-control {
                    border-left: none;
                    padding-left: 0;
                    flex: 1;
                    padding: 6px 12px;
                    border: 1px solid #ccc;
                    border-radius: 0 4px 4px 0;
                }
                
                /* 메타박스 위치 조정 */
                #naver_search_keyword_meta_box {
                    margin-top: 0;
                }
            ');
            
            // 태그 메타박스 바로 아래에 위치하도록 JavaScript 추가
            wp_add_inline_script('postbox', '
                jQuery(document).ready(function($) {
                    // 태그 메타박스 다음에 네이버 검색 키워드 메타박스 이동
                    $("#tagsdiv-post_tag").after($("#naver_search_keyword_meta_box"));
                    
                    // 메타박스가 닫히지 않도록 설정
                    postboxes.add_postbox_toggles(pagenow);
                    $("#naver_search_keyword_meta_box").removeClass("closed");
                });
            ');
        }
    }

    // ===================================
    // 메타박스 관련 기능
    // ===================================

    /**
     * 네이버 검색 키워드 메타박스 추가
     */
    public function add_meta_box() {
        // 태그 메타박스 바로 다음에 추가되도록 설정
        add_meta_box(
            'naver_search_keyword_meta_box',
            '네이버 블로그 검색 키워드',
            array($this, 'render_meta_box'),
            'post',
            'side',
            'core'  // 'core'는 기본 워드프레스 메타박스와 같은 우선순위
        );
    }

    /**
     * 메타박스가 화면 옵션에서 숨겨지지 않도록 설정
     * 
     * @param array $hidden 숨겨진 메타박스 목록
     * @param object $screen 현재 화면 객체
     * @return array 수정된 숨겨진 메타박스 목록
     */
    public function ensure_meta_box_is_visible($hidden, $screen) {
        if ($screen->id === 'post') {
            // hidden 배열에서 우리 메타박스 ID를 제거
            $hidden = array_diff($hidden, array('naver_search_keyword_meta_box'));
        }
        return $hidden;
    }

    /**
     * 메타박스 렌더링
     * 
     * @param object $post 현재 포스트 객체
     */
    public function render_meta_box($post) {
        // Add nonce for security
        wp_nonce_field('naver_search_keyword_nonce', 'naver_search_keyword_nonce');
        
        // Get the current value
        $value = get_post_meta($post->ID, $this->meta_key, true);
        
        // Output the field
        ?>
        <div class="input-group">
            <span class="input-group-text bg-white border-end-0">
                <i class="bi bi-search text-primary"></i>
            </span>
            <input type="text" class="form-control border-start-0" id="naver_search_keyword" 
                   name="naver_search_keyword" value="<?php echo esc_attr($value); ?>" 
                   placeholder="네이버 검색 키워드">
        </div>
        <p class="description">이 키워드로 네이버 블로그 리뷰를 검색합니다. 입력하지 않으면 첫 번째 태그를 사용합니다.</p>
        <?php
    }

    /**
     * 메타박스 데이터 저장
     * 
     * @param int $post_id 현재 포스트 ID
     */
    public function save_meta_box_data($post_id) {
        // Check if our nonce is set
        if (!isset($_POST['naver_search_keyword_nonce'])) {
            return;
        }
        
        // Verify the nonce
        if (!wp_verify_nonce($_POST['naver_search_keyword_nonce'], 'naver_search_keyword_nonce')) {
            return;
        }
        
        // Check user permissions
        if (!current_user_can('edit_post', $post_id)) {
            return;
        }
        
        // Don't save during autosave
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }
        
        // Save the data
        if (isset($_POST['naver_search_keyword'])) {
            $keyword = sanitize_text_field($_POST['naver_search_keyword']);
            update_post_meta($post_id, $this->meta_key, $keyword);
            
            // Clear the cache for this post
            $cache_key = $this->transient_prefix . md5($keyword);
            delete_transient($cache_key);
        }
    }

    // ===================================
    // 네이버 블로그 리뷰 기능
    // ===================================

    /**
     * 포스트 컨텐츠에 네이버 블로그 리뷰 추가
     * 
     * @param string $content 원본 컨텐츠
     * @return string 네이버 블로그 리뷰가 추가된 컨텐츠
     */
    public function append_naver_reviews($content) {
        if (is_single()) {
            $reviews = $this->get_naver_reviews();
            $content .= $reviews;
        }
        return $content;
    }

    /**
     * 네이버 블로그 리뷰 가져오기
     * 
     * @return string 네이버 블로그 리뷰 HTML
     */
    public function get_naver_reviews() {
        // Get keyword from custom field
        $keyword = get_post_meta(get_the_ID(), $this->meta_key, true);
        
        // If no custom keyword is set, fall back to the first tag
        if (empty($keyword)) {
            $post_tags = get_the_tags();
            if (!$post_tags) {
                return '<p>No tags or keyword found for this post. Unable to fetch Naver Blog reviews.</p>';
            }
            $keyword = $post_tags[0]->name;
        }
        
        // Check cache first
        $cache_key = $this->transient_prefix . md5($keyword);
        $cached_result = get_transient($cache_key);
        if ($cached_result !== false) {
            return $cached_result;
        }
        
        $url = "https://openapi.naver.com/v1/search/blog.json?query=" . urlencode($keyword) . "&display=5&sort=sim";
        
        $args = array(
            'headers' => array(
                "X-Naver-Client-Id" => isset($this->options['client_id']) ? $this->options['client_id'] : '',
                "X-Naver-Client-Secret" => isset($this->options['client_secret']) ? $this->options['client_secret'] : ''
            )
        );
        
        $response = wp_remote_get($url, $args);
        
        if (is_wp_error($response)) {
            return '<p>Error fetching reviews from Naver API.</p>';
        }
        
        $body = wp_remote_retrieve_body($response);
        $json = json_decode($body, true);
        
        if (!isset($json['items']) || !is_array($json['items'])) {
            return '<p>Error processing reviews from Naver API.</p>';
        }
        
        $items = $json['items'];
        
        if (empty($items)) {
            return '<p>No reviews found for the keyword: ' . esc_html($keyword) . '.</p>';
        }
        
        $output = '<div class="naver-blog-reviews">';
        $output .= '<h3>관련 네이버 블로그 리뷰: ' . esc_html($keyword) . '</h3>';
        
        foreach ($items as $item) {
            $title = wp_strip_all_tags($item['title']);
            $description = wp_strip_all_tags($item['description']);
            $link = $item['link'];
            $output .= '<div class="naver-blog-review">';
            $output .= '<a href="' . esc_url($link) . '" target="_blank">' . esc_html($title) . '</a>';
            $output .= '<p>' . esc_html($description) . '</p>';
            $output .= '</div>';
        }
        
        $output .= '</div>';
        
        // Cache the result
        set_transient($cache_key, $output, $this->cache_time);
        
        return $output;
    }
}

// ===================================
// 플러그인 인스턴스 생성
// ===================================

new NaverBlogReviews();